<?php
/*
Plugin Name: Simple Google Maps Short Code
Plugin URL: http://pippinsplugins.com/simple-google-maps-short-code
Description: Adds a simple Google Maps short code
Version: 1.2
Author: Pippin Williamson
Author URI: http://pippinsplugins.com
Contributors: mordauk
*/

/**
 * Loads the plugin textdomain
 *
 * @access      private
 * @since       1.2
 * @return      void
*/
function pw_map_textdomain() {

	// Set filter for plugin's languages directory
	$lang_dir = dirname( plugin_basename( __FILE__ ) ) . '/languages/';
	$lang_dir = apply_filters( 'pw_map_languages_directory', $lang_dir );

	// Load the translations
	load_plugin_textdomain( 'simple-google-maps-short-code', false, $lang_dir );
}
add_action( 'init', 'pw_map_textdomain' );

/**
 * Displays the map
 *
 * @access      private
 * @since       1.0
 * @return      void
*/
function pw_map_shortcode( $atts ) {

	$atts = shortcode_atts(
		array(
			'address'           => false,
			'width'             => '100%',
			'height'            => '400px',
			'enablescrollwheel' => 'true',
			'zoom'              => 7,
			'disablecontrols'   => 'false'
		),
		$atts
	);

	$address = $atts['address'];

	if( $address )	if( $address && wp_script_is( 'google-maps-api', 'registered' ) ) :

		wp_print_scripts( 'google-maps-api' );

		$coordinates = pw_map_get_coordinates( $address );

		if( !is_array( $coordinates ) )
			return;

		$map_id = uniqid( 'pw_map_' ); // generate a unique ID for this map

		ob_start(); ?>
        <div id="panel">
            <div id="color-palette"></div>
            <div>
                <button id="delete-button">Удалить Объект</button>
            </div>
        </div>			
		<div class="pw_map_canvas" id="<?php echo esc_attr( $map_id ); ?>" style="height: <?php echo esc_attr( $atts['height'] ); ?>; width: <?php echo esc_attr( $atts['width'] ); ?>"></div>
	    <script src="http://maps.google.com/maps/api/js?sensor=false&libraries=drawing"></script>
		<script type="text/javascript">
var image = new google.maps.MarkerImage("<?php echo plugins_url( 'images/point.png', __FILE__ ); ?>",
	null,
	new google.maps.Point(0,0),
	new google.maps.Point(16,16)
);

var polyOptions = {
	strokeWeight: 0,
	fillOpacity: 0.45,
	editable: true,
	draggable: true
};	

var drawingManager;
var selectedShape;
var colors = ['#1E90FF', '#FF1493', '#32CD32', '#FFFF00', '#4B0082'];
var selectedColor;
var colorButtons = {};

function clearSelection () {
	if (selectedShape) {
		if (selectedShape.type !== 'marker') {
			selectedShape.setEditable(false);
		}
		
		selectedShape = null;
	}
}

function setSelection (shape) {
	if (shape.type !== 'marker') {
		clearSelection();
		shape.setEditable(true);
		selectColor(shape.get('fillColor') || shape.get('strokeColor'));
	}
	
	selectedShape = shape;
}

function deleteSelectedShape () {
	if (selectedShape) {
		selectedShape.setMap(null);
	}
}

function selectColor (color) {
	selectedColor = color;
	for (var i = 0; i < colors.length; ++i) {
		var currColor = colors[i];
		colorButtons[currColor].style.border = currColor == color ? '2px solid #789' : '2px solid #fff';
	}
	// Retrieves the current options from the drawing manager and replaces the
	// stroke or fill color as appropriate.
	var polylineOptions = drawingManager.get('polylineOptions');
	polylineOptions.strokeColor = color;
	drawingManager.set('polylineOptions', polylineOptions);
	var rectangleOptions = drawingManager.get('rectangleOptions');
	rectangleOptions.fillColor = color;
	drawingManager.set('rectangleOptions', rectangleOptions);
	var circleOptions = drawingManager.get('circleOptions');
	circleOptions.fillColor = color;
	drawingManager.set('circleOptions', circleOptions);
	var polygonOptions = drawingManager.get('polygonOptions');
	polygonOptions.fillColor = color;
	drawingManager.set('polygonOptions', polygonOptions);
}

function setSelectedShapeColor (color) {
	if (selectedShape) {
		if (selectedShape.type == google.maps.drawing.OverlayType.POLYLINE) {
			selectedShape.set('strokeColor', color);
		} else {
			selectedShape.set('fillColor', color);
		}
	}
}

function makeColorButton (color) {
	var button = document.createElement('span');
	button.className = 'color-button';
	button.style.backgroundColor = color;
	google.maps.event.addDomListener(button, 'click', function () {
		selectColor(color);
		setSelectedShapeColor(color);
	});
	return button;
}

function buildColorPalette () {
	var colorPalette = document.getElementById('color-palette');
	for (var i = 0; i < colors.length; ++i) {
		var currColor = colors[i];
		var colorButton = makeColorButton(currColor);
		colorPalette.appendChild(colorButton);
		colorButtons[currColor] = colorButton;
	}
	selectColor(colors[0]);
}
	
			var map_<?php echo $map_id; ?>;
			function pw_run_map_<?php echo $map_id ; ?>(){
				var location = new google.maps.LatLng("<?php echo $coordinates['lat']; ?>", "<?php echo $coordinates['lng']; ?>");
				var map_options = {
					zoom: <?php echo $atts['zoom']; ?>,
					center: location,
					scrollwheel: <?php echo 'true' === strtolower( $atts['enablescrollwheel'] ) ? '1' : '0'; ?>,
					disableDefaultUI: <?php echo 'true' === strtolower( $atts['disablecontrols'] ) ? '1' : '0'; ?>,
					mapTypeId: google.maps.MapTypeId.ROADMAP					
				}
				
  var path_zodiac_axis = [
	{lat: 54.7558869004422, lng: 37.081735702939},
    {lat: 55.7558869004422, lng: 37.617634087801},
    {lat: 56.7558869004422, lng: 38.153532472663},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 56.7558869004422, lng: 37.081735702939},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 54.7558869004422, lng: 38.153532472663},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 56.7558869004422, lng: 39.617634087801},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 54.7558869004422, lng: 35.617634087801},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 56.023836092873, lng: 39.617634087801},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 55.487937708011, lng: 35.617634087801},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 55.487937708011, lng: 39.617634087801},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 56.023836092873, lng: 35.617634087801},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 54.7558869004422, lng: 39.617634087801},
	{lat: 55.7558869004422, lng: 37.617634087801},
	{lat: 56.7558869004422, lng: 35.617634087801},
  ];
  var zodiac_axis = new google.maps.Polyline({
    path: path_zodiac_axis,
	draggable: true,
    geodesic: true,
    strokeColor: '#FF0000',
    strokeOpacity: 1.0,
    strokeWeight: 2
  });
map_<?php echo $map_id ; ?> = new google.maps.Map(document.getElementById("<?php echo $map_id ; ?>"), map_options);
zodiac_axis.setMap(map_<?php echo $map_id ; ?>); 

drawingManager = new google.maps.drawing.DrawingManager({
drawingMode: google.maps.drawing.OverlayType.MARKER,
drawingControl: true,
drawingControlOptions: {
  position: google.maps.ControlPosition.TOP_CENTER,
  drawingModes: [
	google.maps.drawing.OverlayType.MARKER,
	google.maps.drawing.OverlayType.CIRCLE,
	google.maps.drawing.OverlayType.POLYGON,
	google.maps.drawing.OverlayType.POLYLINE,
	google.maps.drawing.OverlayType.RECTANGLE
  ]
},
markerOptions: {
	icon: image, 
	draggable: true
},
polylineOptions: {
	editable: true,
	draggable: true
},	
rectangleOptions: polyOptions,
circleOptions: polyOptions,
polygonOptions: polyOptions,
});
  
drawingManager.setMap(map_<?php echo $map_id ; ?>);	
			}
			
			pw_run_map_<?php echo $map_id ; ?>();

			
			
               google.maps.event.addListener(drawingManager, 'overlaycomplete', function (e) {
                    var newShape = e.overlay;
                    newShape.type = e.type;
                    
                    if (e.type !== google.maps.drawing.OverlayType.MARKER) {
                        // Switch back to non-drawing mode after drawing a shape.
                        drawingManager.setDrawingMode(null);
                        // Add an event listener that selects the newly-drawn shape when the user
                        // mouses down on it.
                        google.maps.event.addListener(newShape, 'click', function (e) {
                            if (e.vertex !== undefined) {
                                if (newShape.type === google.maps.drawing.OverlayType.POLYGON) {
                                    var path = newShape.getPaths().getAt(e.path);
                                    path.removeAt(e.vertex);
                                    if (path.length < 3) {
                                        newShape.setMap(null);
                                    }
                                }
                                if (newShape.type === google.maps.drawing.OverlayType.POLYLINE) {
                                    var path = newShape.getPath();
                                    path.removeAt(e.vertex);
                                    if (path.length < 2) {
                                        newShape.setMap(null);
                                    }
                                }
                            }
                            setSelection(newShape);
                        });
                        setSelection(newShape);
                    }
                    else {
                        google.maps.event.addListener(newShape, 'click', function (e) {
                            setSelection(newShape);
                        });
                        setSelection(newShape);
                    }
                });				
                // Clear the current selection when the drawing mode is changed, or when the
                // map is clicked.
                google.maps.event.addListener(drawingManager, 'drawingmode_changed', clearSelection);
                google.maps.event.addListener(map_<?php echo $map_id ; ?>, 'click', clearSelection);
                google.maps.event.addDomListener(document.getElementById('delete-button'), 'click', deleteSelectedShape);
                buildColorPalette();
 			
		</script>		
		<?php
	
		return ob_get_clean();
	else :
		return __( 'This Google Map cannot be loaded because the maps API does not appear to be loaded', 'simple-google-maps-short-code' );
	endif;
}
add_shortcode( 'pw_map', 'pw_map_shortcode' );


/**
 * Loads Google Map API
 *
 * @access      private
 * @since       1.0
 * @return      void
*/
function pw_map_load_scripts() {
	wp_register_script( 'google-maps-api', '//maps.google.com/maps/api/js?sensor=false' );
}
add_action( 'wp_enqueue_scripts', 'pw_map_load_scripts' );



/**
 * Retrieve coordinates for an address
 *
 * Coordinates are cached using transients and a hash of the address
 *
 * @access      private
 * @since       1.0
 * @return      void
*/
function pw_map_get_coordinates( $address, $force_refresh = false ) {

    $address_hash = md5( $address );

    $coordinates = get_transient( $address_hash );

    if ($force_refresh || $coordinates === false) {

    	$args       = array( 'address' => urlencode( $address ), 'sensor' => 'false' );
    	$url        = add_query_arg( $args, 'http://maps.googleapis.com/maps/api/geocode/json' );
     	$response 	= wp_remote_get( $url );

     	if( is_wp_error( $response ) )
     		return;

     	$data = wp_remote_retrieve_body( $response );

     	if( is_wp_error( $data ) )
     		return;

		if ( $response['response']['code'] == 200 ) {

			$data = json_decode( $data );

			if ( $data->status === 'OK' ) {

			  	$coordinates = $data->results[0]->geometry->location;

			  	$cache_value['lat'] 	= $coordinates->lat;
			  	$cache_value['lng'] 	= $coordinates->lng;
			  	$cache_value['address'] = (string) $data->results[0]->formatted_address;

			  	// cache coordinates for 3 months
			  	set_transient($address_hash, $cache_value, 3600*24*30*3);
			  	$data = $cache_value;

			} elseif ( $data->status === 'ZERO_RESULTS' ) {
			  	return __( 'No location found for the entered address.', 'simple-google-maps-short-code' );
			} elseif( $data->status === 'INVALID_REQUEST' ) {
			   	return __( 'Invalid request. Did you enter an address?', 'simple-google-maps-short-code' );
			} else {
				return __( 'Something went wrong while retrieving your map, please ensure you have entered the short code correctly.', 'simple-google-maps-short-code' );
			}

		} else {
		 	return __( 'Unable to contact Google API service.', 'simple-google-maps-short-code' );
		}

    } else {
       // return cached results
       $data = $coordinates;
    }

    return $data;
}


/**
 * Fixes a problem with responsive themes
 *
 * @access      private
 * @since       1.0.1
 * @return      void
*/

function pw_map_css() {
	echo '<style type="text/css">
	/* =Responsive Map fix
-------------------------------------------------------------- */

#panel {
	width: 100%;
	text-align: right;
	font-family: Arial, sans-serif;
	font-size: 13px;
	margin-bottom: 10px;
}
#color-palette {
	clear: both;
}

#delete-button {
	margin-top: 5px;
}

.pw_map_canvas img {
	max-width: none;
}
.color-button {
	width: 14px;
	height: 14px;
	font-size: 0;
	margin: 2px;
	float: left;
	cursor: pointer;
}
</style>';

}
add_action( 'wp_head', 'pw_map_css' );